@@ -50,17 +50,9 @@
 struct NautilusTreeViewDetails {
 	GtkWidget *scrolled_window;
 	GtkTreeView *tree_widget;
-#if SORT_MODEL_WORKS
 	GtkTreeModelSort *sort_model;
-#else
-	NautilusTreeModel *sort_model;
-#endif
 	NautilusTreeModel *child_model;
 
-	gboolean show_hidden_files;
-	gboolean show_backup_files;
-	gboolean show_only_directories;
-
 	NautilusFile *activation_file;
 	guint save_expansion_state_idle_id;
 };
@@ -88,13 +80,10 @@ load_expansion_state (NautilusTreeView *view)
 static NautilusFile *
 sort_model_iter_to_file (NautilusTreeView *view, GtkTreeIter *iter)
 {
-	GtkTreeIter copy;
+	GtkTreeIter child_iter;
 
-	copy = *iter;
-#if SORT_MODEL_WORKS
-	gtk_tree_model_sort_convert_iter_to_child_iter (view->details->sort_model, &copy, &copy);
-#endif
-	return nautilus_tree_model_iter_get_file (view->details->child_model, &copy);
+	gtk_tree_model_sort_convert_iter_to_child_iter (view->details->sort_model, &child_iter, iter);
+	return nautilus_tree_model_iter_get_file (view->details->child_model, &child_iter);
 }
 
 static NautilusFile *
@@ -224,18 +213,12 @@ selection_changed_callback (GtkTreeSelection *selection,
 	g_list_free (attrs);
 }
 
-#if SORT_MODEL_WORKS
-
 static int
 compare_rows (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer callback_data)
 {
-	NautilusTreeView *view;
 	NautilusFile *file_a, *file_b;
-	char *uri_a, *uri_b;
 	int result;
 
-	view = NAUTILUS_TREE_VIEW (callback_data);
-
 	file_a = nautilus_tree_model_iter_get_file (NAUTILUS_TREE_MODEL (model), a);
 	file_b = nautilus_tree_model_iter_get_file (NAUTILUS_TREE_MODEL (model), b);
 
@@ -246,11 +229,6 @@ compare_rows (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer call
 	} else if (file_b == NULL) {
 		result = +1;
 	} else {
-		uri_a = nautilus_file_get_uri (file_a);
-		uri_b = nautilus_file_get_uri (file_b);
-		g_message ("comparing %s with %s", uri_a, uri_b);
-		g_free (uri_a);
-		g_free (uri_b);
 		result = nautilus_file_compare_for_sort (file_a, file_b,
 							 NAUTILUS_FILE_SORT_BY_DISPLAY_NAME,
 							 FALSE, FALSE);
@@ -262,8 +240,6 @@ compare_rows (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer call
 	return result;
 }
 
-#endif
-
 static void
 create_tree (NautilusTreeView *view)
 {
@@ -271,19 +247,15 @@ create_tree (NautilusTreeView *view)
 	GtkCellRenderer *cell;
 	
 	view->details->child_model = nautilus_tree_model_new ("file:///");
-#if SORT_MODEL_WORKS
-	view->details->sort_model = GTK_TREE_MODEL_SORT (gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (view->details->child_model)));
+	view->details->sort_model = GTK_TREE_MODEL_SORT
+		(gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (view->details->child_model)));
 	g_object_unref (view->details->child_model);
-#else
-	view->details->sort_model = view->details->child_model;
-#endif
-	view->details->tree_widget = GTK_TREE_VIEW (gtk_tree_view_new_with_model (GTK_TREE_MODEL (view->details->sort_model)));
+	view->details->tree_widget = GTK_TREE_VIEW
+		(gtk_tree_view_new_with_model (GTK_TREE_MODEL (view->details->sort_model)));
 	g_object_unref (view->details->sort_model);
 
-#if SORT_MODEL_WORKS
 	gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (view->details->sort_model),
 						 compare_rows, view, NULL);
-#endif
 
 	gtk_tree_view_set_headers_visible (view->details->tree_widget, FALSE);
 
@@ -312,14 +284,31 @@ create_tree (NautilusTreeView *view)
 	gtk_container_add (GTK_CONTAINER (view->details->scrolled_window),
 			   GTK_WIDGET (view->details->tree_widget));
 
-	g_signal_connect_swapped (view->details->tree_widget, "row_expanded",
-				  G_CALLBACK (schedule_save_expansion_state_callback), view);
-	g_signal_connect_swapped (view->details->tree_widget, "row_collapsed",
-				  G_CALLBACK (schedule_save_expansion_state_callback), view);
+	g_signal_connect_object (view->details->tree_widget, "row_expanded",
+				 G_CALLBACK (schedule_save_expansion_state_callback), view, G_CONNECT_SWAPPED);
+	g_signal_connect_object (view->details->tree_widget, "row_collapsed",
+				 G_CALLBACK (schedule_save_expansion_state_callback), view, G_CONNECT_SWAPPED);
 
-	g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (view->details->tree_widget)),
-			  "changed",
-			  G_CALLBACK (selection_changed_callback), view);
+	g_signal_connect_object (gtk_tree_view_get_selection (GTK_TREE_VIEW (view->details->tree_widget)), "changed",
+				 G_CALLBACK (selection_changed_callback), view, 0);
+}
+
+static void
+update_filtering_from_preferences (NautilusTreeView *view)
+{
+	if (view->details->child_model == NULL) {
+		return;
+	}
+
+	nautilus_tree_model_set_show_hidden_files
+		(view->details->child_model,
+		 eel_preferences_get_boolean (NAUTILUS_PREFERENCES_SHOW_HIDDEN_FILES));
+	nautilus_tree_model_set_show_backup_files
+		(view->details->child_model,
+		 eel_preferences_get_boolean (NAUTILUS_PREFERENCES_SHOW_BACKUP_FILES));
+	nautilus_tree_model_set_show_only_directories
+		(view->details->child_model,
+		 eel_preferences_get_boolean (NAUTILUS_PREFERENCES_TREE_SHOW_ONLY_DIRECTORIES));
 }
 
 static void
@@ -330,45 +319,16 @@ tree_activate_callback (BonoboControl *control, gboolean activating, gpointer us
 	view = NAUTILUS_TREE_VIEW (user_data);
 
 	if (activating && view->details->tree_widget == NULL) {
-		create_tree (view);
 		load_expansion_state (view);
+		create_tree (view);
+		update_filtering_from_preferences (view);
 	}
 }
 
-static int
-get_filtering_as_integer (NautilusTreeView *view)
-{
-	return (((view->details->show_hidden_files << 1)
-		 | view->details->show_backup_files) << 1)
-		| view->details->show_only_directories;
-}
-
-static void
-update_filtering_from_preferences (NautilusTreeView *view)
-{
-	view->details->show_hidden_files =
-		eel_preferences_get_boolean (NAUTILUS_PREFERENCES_SHOW_HIDDEN_FILES);
-	view->details->show_backup_files =
-		eel_preferences_get_boolean (NAUTILUS_PREFERENCES_SHOW_BACKUP_FILES);
-	view->details->show_only_directories =
-		eel_preferences_get_boolean (NAUTILUS_PREFERENCES_TREE_SHOW_ONLY_DIRECTORIES);
-}
-
 static void
 filtering_changed_callback (gpointer callback_data)
 {
-	NautilusTreeView *view;
-	int filtering_before;
-
-	view = NAUTILUS_TREE_VIEW (callback_data);
-
-	filtering_before = get_filtering_as_integer (view);
-	update_filtering_from_preferences (view);
-	if (filtering_before == get_filtering_as_integer (view)) {
-		return;
-	}
-	
-	/* FIXME: reload the whole tree */
+	update_filtering_from_preferences (NAUTILUS_TREE_VIEW (callback_data));
 }
 
 static void
@@ -398,8 +358,6 @@ nautilus_tree_view_instance_init (NautilusTreeView *view)
 				      filtering_changed_callback, view);
 	eel_preferences_add_callback (NAUTILUS_PREFERENCES_TREE_SHOW_ONLY_DIRECTORIES,
 				      filtering_changed_callback, view);
-
-	update_filtering_from_preferences (view);
 }
 
 static void
