@@ -25,6 +25,7 @@
 */
 
 #include <config.h>
+#include "fm-icon-container.h"
 #include "fm-desktop-icon-view.h"
 
 #include <X11/Xatom.h>
@@ -84,7 +85,6 @@ struct FMDesktopIconViewDetails
 	/* For the desktop rescanning
 	 */
 	gulong delayed_init_signal;
-	gulong done_loading_signal;
 	guint reload_desktop_timeout;
 	gboolean pending_rescan;
 };
@@ -100,16 +100,13 @@ static void     fm_desktop_icon_view_trash_state_changed_callback (NautilusTrash
 								   gboolean                state,
 								   gpointer                callback_data);
 static void     home_uri_changed                                  (gpointer                user_data);
+static void     default_zoom_level_changed                        (gpointer                user_data);
 static void     volume_mounted_callback                           (NautilusVolumeMonitor  *monitor,
 								   NautilusVolume         *volume,
 								   FMDesktopIconView      *icon_view);
 static void     volume_unmounted_callback                         (NautilusVolumeMonitor  *monitor,
 								   NautilusVolume         *volume,
 								   FMDesktopIconView      *icon_view);
-static int      desktop_icons_compare_callback                    (NautilusIconContainer  *container,
-								   NautilusFile           *file_a,
-								   NautilusFile           *file_b,
-								   FMDesktopIconView      *icon_view);
 static void     delete_all_mount_links                            (void);
 static void     update_home_link_and_delete_copies                (void);
 static void     update_trash_link_and_delete_copies               (void);
@@ -235,23 +232,16 @@ fm_desktop_icon_view_finalize (GObject *object)
 		gtk_timeout_remove (icon_view->details->reload_desktop_timeout);
 	}
 
-	if (icon_view->details->done_loading_signal != 0) {
-		g_signal_handler_disconnect (fm_directory_view_get_model (FM_DIRECTORY_VIEW (icon_view)),
-					     icon_view->details->done_loading_signal);
-	}
-
-	if (icon_view->details->delayed_init_signal != 0) {
-		g_signal_handler_disconnect (icon_view,
-					     icon_view->details->delayed_init_signal);
-	}
-	
 	/* Delete all of the link files. */
 	delete_all_mount_links ();
 	
 	eel_preferences_remove_callback (NAUTILUS_PREFERENCES_HOME_URI,
 					 home_uri_changed,
 					 icon_view);
-
+	eel_preferences_remove_callback (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL,
+					 default_zoom_level_changed,
+					 icon_view);
+	
 	/* Clean up details */	
 	if (icon_view->details->ui != NULL) {
 		bonobo_ui_component_unset_container (icon_view->details->ui, NULL);
@@ -466,6 +456,35 @@ event_callback (GtkWidget *widget, GdkEvent *event, FMDesktopIconView *desktop_i
 {
 }
 
+
+static NautilusZoomLevel
+get_default_zoom_level (void)
+{
+	static gboolean auto_storage_added = FALSE;
+	static NautilusZoomLevel default_zoom_level = NAUTILUS_ZOOM_LEVEL_STANDARD;
+
+	if (!auto_storage_added) {
+		auto_storage_added = TRUE;
+		eel_preferences_add_auto_enum (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL,
+					       (int *) &default_zoom_level);
+	}
+
+	return CLAMP (default_zoom_level, NAUTILUS_ZOOM_LEVEL_SMALLEST, NAUTILUS_ZOOM_LEVEL_LARGEST);
+}
+
+static void
+default_zoom_level_changed (gpointer user_data)
+{
+	NautilusZoomLevel new_level;
+	FMDesktopIconView *desktop_icon_view;
+
+	desktop_icon_view = FM_DESKTOP_ICON_VIEW (user_data);
+	new_level = get_default_zoom_level ();
+
+	nautilus_icon_container_set_zoom_level (get_icon_container (desktop_icon_view),
+						new_level);
+}
+
 /* Update home link to point to new home uri */
 static void
 home_uri_changed (gpointer callback_data)
@@ -521,10 +540,9 @@ static void
 delayed_init (FMDesktopIconView *desktop_icon_view)
 {
 	/* Keep track of the load time. */
-	desktop_icon_view->details->done_loading_signal = 
-		g_signal_connect (fm_directory_view_get_model (FM_DIRECTORY_VIEW (desktop_icon_view)),
-				  "done_loading",
-				  G_CALLBACK (done_loading), desktop_icon_view);
+	g_signal_connect_object (fm_directory_view_get_model (FM_DIRECTORY_VIEW (desktop_icon_view)),
+				 "done_loading",
+				 G_CALLBACK (done_loading), desktop_icon_view, 0);
 
 	/* Monitor desktop directory. */
 	desktop_icon_view->details->reload_desktop_timeout =
@@ -536,6 +554,8 @@ delayed_init (FMDesktopIconView *desktop_icon_view)
 	desktop_icon_view->details->delayed_init_signal = 0;
 }
 
+
+
 static void
 fm_desktop_icon_view_init (FMDesktopIconView *desktop_icon_view)
 {
@@ -553,6 +573,8 @@ fm_desktop_icon_view_init (FMDesktopIconView *desktop_icon_view)
 
 	icon_container = get_icon_container (desktop_icon_view);
 
+	fm_icon_container_set_sort_desktop (FM_ICON_CONTAINER (icon_container), TRUE);
+
 	/* Set up details */
 	desktop_icon_view->details = g_new0 (FMDesktopIconViewDetails, 1);	
 
@@ -560,13 +582,13 @@ fm_desktop_icon_view_init (FMDesktopIconView *desktop_icon_view)
 	 * way to keep track of the items on the desktop.
 	 */
 	if (!nautilus_monitor_active ()) {
-		desktop_icon_view->details->delayed_init_signal = g_signal_connect
+		desktop_icon_view->details->delayed_init_signal = g_signal_connect_object
 			(desktop_icon_view, "begin_loading",
-			 G_CALLBACK (delayed_init), desktop_icon_view);
+			 G_CALLBACK (delayed_init), desktop_icon_view, 0);
 	}
 	
 	nautilus_icon_container_set_is_fixed_size (icon_container, TRUE);
-	
+
 	/* Set up default mount black list */
 	list = g_list_prepend (NULL, g_strdup ("/proc"));
 	list = g_list_prepend (list, g_strdup ("/boot"));
@@ -576,6 +598,7 @@ fm_desktop_icon_view_init (FMDesktopIconView *desktop_icon_view)
 	allocation = &GTK_WIDGET (icon_container)->allocation;
 	allocation->x = 0;
 	allocation->y = 0;
+	
 	gtk_widget_queue_resize (GTK_WIDGET (icon_container));
 
 	hadj = GTK_LAYOUT (icon_container)->hadjustment;
@@ -600,21 +623,14 @@ fm_desktop_icon_view_init (FMDesktopIconView *desktop_icon_view)
 					     	     create_one_mount_link,
 						     desktop_icon_view);
 	
-	g_signal_connect (icon_container, "middle_click",
-			  G_CALLBACK (fm_desktop_icon_view_handle_middle_click), desktop_icon_view);
-			    
-	g_signal_connect (icon_container, "compare_icons",
-			  G_CALLBACK (desktop_icons_compare_callback), desktop_icon_view);
-
-	g_signal_connect (desktop_icon_view, "event",
-			  G_CALLBACK (event_callback), desktop_icon_view);
-	
+	g_signal_connect_object (icon_container, "middle_click",
+				 G_CALLBACK (fm_desktop_icon_view_handle_middle_click), desktop_icon_view, 0);
+	g_signal_connect_object (desktop_icon_view, "event",
+				 G_CALLBACK (event_callback), desktop_icon_view, 0);
 	g_signal_connect_object (nautilus_trash_monitor_get (), "trash_state_changed",
-				 G_CALLBACK (fm_desktop_icon_view_trash_state_changed_callback), desktop_icon_view, 0);
-	
+				 G_CALLBACK (fm_desktop_icon_view_trash_state_changed_callback), desktop_icon_view, 0);	
 	g_signal_connect_object (nautilus_volume_monitor_get (), "volume_mounted",
 				 G_CALLBACK (volume_mounted_callback), desktop_icon_view, 0);
-	
 	g_signal_connect_object (nautilus_volume_monitor_get (), "volume_unmounted",
 				 G_CALLBACK (volume_unmounted_callback), desktop_icon_view, 0);
 	
@@ -622,6 +638,11 @@ fm_desktop_icon_view_init (FMDesktopIconView *desktop_icon_view)
 				      home_uri_changed,
 				      desktop_icon_view);
 
+	eel_preferences_add_callback (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL,
+				      default_zoom_level_changed,
+				      desktop_icon_view);
+	default_zoom_level_changed (desktop_icon_view);
+
 	/* Read out the panel desktop area and update the icon container
 	 * accordingly */
 	panel_desktop_area_changed (desktop_icon_view);
@@ -638,6 +659,21 @@ new_terminal_callback (BonoboUIComponent *component, gpointer data, const char *
 	eel_gnome_open_terminal (NULL);
 }
 
+static void
+new_launcher_callback (BonoboUIComponent *component, gpointer data, const char *verb)
+{
+	char *desktop_directory;
+	
+	desktop_directory = nautilus_get_desktop_directory ();
+
+	nautilus_launch_application_from_command ("gnome-desktop-item-edit", 
+						  "gnome-desktop-item-edit --create-new",
+						  desktop_directory, 
+						  FALSE);
+	g_free (desktop_directory);
+
+}
+
 static void
 change_background_callback (BonoboUIComponent *component, 
 	  		    gpointer data, 
@@ -645,7 +681,7 @@ change_background_callback (BonoboUIComponent *component,
 {
 	nautilus_launch_application_from_command 
 		(_("Background"),
-		 "gnome2-background-properties", NULL, FALSE);
+		 "gnome-background-properties", NULL, FALSE);
 }
 
 static void
@@ -896,7 +932,8 @@ update_home_link_and_delete_copies (void)
 	
 	if (!update_link_and_delete_copies (nautilus_link_local_is_home_link,
 					    NULL,
-					    home_uri)) {
+					    home_uri)
+ 	    && !eel_preferences_get_boolean (NAUTILUS_PREFERENCES_DESKTOP_IS_HOME_DIR)) {
 		nautilus_link_local_create (desktop_directory,
 					    home_link_name,
 					    "desktop-home", 
@@ -939,93 +976,6 @@ delete_all_mount_links (void)
 				       "", "");
 }
 
-static char *
-get_local_path (NautilusFile *file)
-{
-	char *uri, *local_path;
-
-	uri = nautilus_file_get_uri (file);
-	local_path = gnome_vfs_get_local_path_from_uri (uri);
-	g_free (uri);
-	return local_path;
-}
-
-/* Sort as follows:
- *   1) home link
- *   2) mount links
- *   3) other
- *   4) trash link
- */
-
-typedef enum {
-	SORT_HOME_LINK,
-	SORT_MOUNT_LINK,
-	SORT_OTHER,
-	SORT_TRASH_LINK
-} SortCategory;
-
-static SortCategory
-get_sort_category (NautilusFile *file)
-{
-	char *path;
-	SortCategory category;
-
-	if (!nautilus_file_is_nautilus_link (file)) {
-		category = SORT_OTHER;
-	} else {
-		path = get_local_path (file);
-		g_return_val_if_fail (path != NULL, SORT_OTHER);
-		
-		switch (nautilus_link_local_get_link_type (path)) {
-		case NAUTILUS_LINK_HOME:
-			category = SORT_HOME_LINK;
-			break;
-		case NAUTILUS_LINK_MOUNT:
-			category = SORT_MOUNT_LINK;
-			break;
-		case NAUTILUS_LINK_TRASH:
-			category = SORT_TRASH_LINK;
-			break;
-		default:
-			category = SORT_OTHER;
-			break;
-		}
-		
-		g_free (path);
-	}
-	
-	return category;
-}
-
-static int
-desktop_icons_compare_callback (NautilusIconContainer *container,
-				NautilusFile *file_a,
-				NautilusFile *file_b,
-				FMDesktopIconView *icon_view)
-{
-	SortCategory category_a, category_b;
-	
-	category_a = get_sort_category (file_a);
-	category_b = get_sort_category (file_b);
-
-	if (category_a == category_b) {
-		return nautilus_file_compare_for_sort 
-			(file_a, file_b, NAUTILUS_FILE_SORT_BY_DISPLAY_NAME, 
-			 fm_directory_view_should_sort_directories_first (FM_DIRECTORY_VIEW (icon_view)), 
-			 FALSE);
-	}
-
-	/* We know the answer, so prevent the other handlers
-	 * from overwriting our result.
-	 */
-	g_signal_stop_emission_by_name (container, "compare_icons");
-	if (category_a < category_b) {
-		return -1;
-	} else {
-		return +1;
-	}
-}
-
 static MountParameters *
 mount_parameters_new (FMDesktopIconView *view, const char *mount_path)
 {
@@ -1216,6 +1166,7 @@ real_merge_menus (FMDirectoryView *view)
 		BONOBO_UI_VERB ("Change Background", change_background_callback),
 		BONOBO_UI_VERB ("Empty Trash Conditional", empty_trash_callback),
 		BONOBO_UI_VERB ("New Terminal", new_terminal_callback),
+		BONOBO_UI_VERB ("New Launcher", new_launcher_callback),
 		BONOBO_UI_VERB ("Reset Background", reset_background_callback),
 		BONOBO_UI_VERB ("Unmount Volume Conditional", unmount_volume_callback),
 		BONOBO_UI_VERB_END
