@@ -26,12 +26,15 @@
 #include "fm-list-model.h"
 
 #include <string.h>
+#include <eel/eel-gtk-macros.h>
 #include <gtk/gtktreednd.h>
 #include <gtk/gtktreesortable.h>
 #include <libnautilus-private/nautilus-icon-factory.h>
 
 #define G_SLIST(x) ((GSList *) x)
 
+static GObjectClass *parent_class;
+
 struct FMListModelDetails {
 	GSList *files;
 	GSList *tail;
@@ -525,9 +528,8 @@ fm_list_model_add_file (FMListModel *model, NautilusFile *file)
 	else {
 		for (tmp = model->details->files; tmp; tmp = tmp->next) {
 			file1 = tmp->data;
-			
-			
-			result = nautilus_file_compare_for_sort (file1, file,
+
+			result = nautilus_file_compare_for_sort (file, file1,
 								 fm_list_model_get_nautilus_file_sort_type (model),
 								 model->details->sort_directories_first,
 								 (model->details->order == GTK_SORT_DESCENDING));
@@ -712,12 +714,12 @@ fm_list_model_get_sort_column_id_from_sort_type (NautilusFileSortType sort_type)
 	switch (sort_type) {
 	case NAUTILUS_FILE_SORT_NONE:
 		return -1;
-	case NAUTILUS_FILE_SORT_BY_SIZE:
+	case NAUTILUS_FILE_SORT_BY_DISPLAY_NAME:
 		return FM_LIST_MODEL_NAME_COLUMN;
 	case NAUTILUS_FILE_SORT_BY_TYPE:
 		return FM_LIST_MODEL_TYPE_COLUMN;
-	case NAUTILUS_FILE_SORT_BY_DISPLAY_NAME:
-		return FM_LIST_MODEL_NAME_COLUMN;
+	case NAUTILUS_FILE_SORT_BY_SIZE:
+		return FM_LIST_MODEL_SIZE_COLUMN;
 	case NAUTILUS_FILE_SORT_BY_MTIME:
 		return FM_LIST_MODEL_DATE_MODIFIED_COLUMN;
 	case NAUTILUS_FILE_SORT_BY_EMBLEMS:
@@ -728,6 +730,18 @@ fm_list_model_get_sort_column_id_from_sort_type (NautilusFileSortType sort_type)
 	g_return_val_if_reached (-1);
 }
 
+static void
+fm_list_model_finalize (GObject *object)
+{
+	FMListModel *model;
+
+	model = FM_LIST_MODEL (object);
+
+	g_free (model->details);
+	
+	EEL_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
+}
+
 static void
 fm_list_model_init (FMListModel *model)
 {
@@ -739,6 +753,12 @@ fm_list_model_init (FMListModel *model)
 static void
 fm_list_model_class_init (FMListModelClass *klass)
 {
+	GObjectClass *object_class;
+
+	object_class = (GObjectClass *)klass;
+	parent_class = g_type_class_peek_parent (klass);
+
+	object_class->finalize = fm_list_model_finalize;
 }
 
 static void
