@@ -45,7 +45,8 @@ struct FMListViewDetails {
 	GtkTreeView *tree_view;
 	FMListModel *model;
 
-	GtkTreeViewColumn *file_name_column;
+	GtkTreeViewColumn   *file_name_column;
+	GtkCellRendererText *file_name_cell;
 };
 
 static GtkTargetEntry drag_types [] = {
@@ -86,10 +87,12 @@ list_activate_callback (GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewCo
 	GList *file_list;
 	
 	view = FM_DIRECTORY_VIEW (user_data);
-	
-	file_list = fm_list_view_get_selection (view);
-	fm_directory_view_activate_files (view, file_list);
-	nautilus_file_list_free (file_list);
+
+	if (eel_preferences_get_enum (NAUTILUS_PREFERENCES_CLICK_POLICY) == NAUTILUS_CLICK_POLICY_DOUBLE) {
+		file_list = fm_list_view_get_selection (view);
+		fm_directory_view_activate_files (view, file_list);
+		nautilus_file_list_free (file_list);
+	}
 }
 
 /* Move these to eel? */
@@ -124,16 +127,18 @@ tree_view_has_selection (GtkTreeView *view)
 static void
 event_after_callback (GtkWidget *widget, GdkEventAny *event, gpointer callback_data)
 {
-	/* Put up the right kind of menu if we right click in the tree view. */
+	FMDirectoryView *view;
+
+	view = FM_DIRECTORY_VIEW (callback_data);
+
 	if (event->type == GDK_BUTTON_PRESS
 	    && event->window == gtk_tree_view_get_bin_window (GTK_TREE_VIEW (widget))
-	    && ((GdkEventButton *) event)->button == 3) {
+	    && (((GdkEventButton *) event)->button == 3)) {
+		/* Put up the right kind of menu if we right click in the tree view. */
 		if (tree_view_has_selection (GTK_TREE_VIEW (widget))) {
-			fm_directory_view_pop_up_selection_context_menu
-				(FM_DIRECTORY_VIEW (callback_data), (GdkEventButton *) event);
+			fm_directory_view_pop_up_selection_context_menu (view, (GdkEventButton *) event);
 		} else {
-			fm_directory_view_pop_up_background_context_menu
-				(FM_DIRECTORY_VIEW (callback_data), (GdkEventButton *) event);
+			fm_directory_view_pop_up_background_context_menu (view, (GdkEventButton *) event);
 		}
 	}
 }
@@ -152,6 +157,26 @@ button_press_callback (GtkWidget *widget, GdkEventButton *event, gpointer callba
 	return FALSE;
 }
 
+static gboolean
+button_release_callback (GtkWidget *widget, GdkEventButton *event, gpointer callback_data)
+{
+	FMDirectoryView *view;
+	GList *file_list;
+
+	view = FM_DIRECTORY_VIEW (callback_data);
+
+	if (event->window == gtk_tree_view_get_bin_window (GTK_TREE_VIEW (widget)) &&
+	    event->button == 1 &&
+	    eel_preferences_get_enum (NAUTILUS_PREFERENCES_CLICK_POLICY) == NAUTILUS_CLICK_POLICY_SINGLE) {
+		/* Handle single click activation preference. */
+		file_list = fm_list_view_get_selection (view);
+		fm_directory_view_activate_files (view, file_list);
+		nautilus_file_list_free (file_list);
+	}
+
+	return FALSE;
+}
+
 static void
 cell_renderer_edited (GtkCellRendererText *cell,
 		      const char          *path_str,
@@ -179,6 +204,24 @@ cell_renderer_edited (GtkCellRendererText *cell,
 	nautilus_file_unref (file);
 }
 
+static void
+click_policy_changed (gpointer callback_data)
+{
+	FMListView *view;
+
+	view = FM_LIST_VIEW (callback_data);
+
+	if (eel_preferences_get_enum (NAUTILUS_PREFERENCES_CLICK_POLICY) == NAUTILUS_CLICK_POLICY_SINGLE) {
+		g_object_set (G_OBJECT (view->details->file_name_cell),
+			      "underline", PANGO_UNDERLINE_SINGLE,
+			      NULL);
+	} else {
+		g_object_set (G_OBJECT (view->details->file_name_cell),
+			      "underline", PANGO_UNDERLINE_NONE,
+			      NULL);
+	}
+}
+
 static void
 create_and_set_up_tree_view (FMListView *view)
 {
@@ -196,7 +239,9 @@ create_and_set_up_tree_view (FMListView *view)
 				 G_CALLBACK (event_after_callback), view, 0);
 	g_signal_connect_object (view->details->tree_view, "button_press_event",
 				 G_CALLBACK (button_press_callback), view, 0);
-
+	g_signal_connect_object (view->details->tree_view, "button_release_event",
+				 G_CALLBACK (button_release_callback), view, 0);
+	
 	view->details->model = g_object_new (FM_TYPE_LIST_MODEL, NULL);
 	gtk_tree_view_set_model (view->details->tree_view, GTK_TREE_MODEL (view->details->model));
 	g_object_unref (view->details->model);
@@ -213,6 +258,7 @@ create_and_set_up_tree_view (FMListView *view)
 	view->details->file_name_column = gtk_tree_view_column_new ();
 	gtk_tree_view_column_set_sort_column_id (view->details->file_name_column, FM_LIST_MODEL_NAME_COLUMN);
 	gtk_tree_view_column_set_title (view->details->file_name_column, _("File name"));
+	gtk_tree_view_column_set_resizable (view->details->file_name_column, TRUE);
 
 	gtk_tree_view_column_pack_start (view->details->file_name_column, cell, FALSE);
 	gtk_tree_view_column_set_attributes (view->details->file_name_column, cell,
@@ -221,7 +267,7 @@ create_and_set_up_tree_view (FMListView *view)
 
 	cell = gtk_cell_renderer_text_new ();
 	g_signal_connect (cell, "edited", G_CALLBACK (cell_renderer_edited), view);
-	
+
 	gtk_tree_view_column_pack_start (view->details->file_name_column, cell, TRUE);
 	gtk_tree_view_column_set_attributes (view->details->file_name_column, cell,
 					     "text", FM_LIST_MODEL_NAME_COLUMN,
@@ -229,6 +275,12 @@ create_and_set_up_tree_view (FMListView *view)
 					     NULL);
 	gtk_tree_view_append_column (view->details->tree_view, view->details->file_name_column);
 
+	/* Set up underline of file name. */
+	view->details->file_name_cell = (GtkCellRendererText *)cell;
+	eel_preferences_add_callback (NAUTILUS_PREFERENCES_CLICK_POLICY,
+				      click_policy_changed, view);
+	click_policy_changed (view);
+
 	/* Create the size column */
 	cell = gtk_cell_renderer_text_new ();
 	g_object_set (G_OBJECT (cell),
@@ -238,6 +290,7 @@ create_and_set_up_tree_view (FMListView *view)
 							   cell,
 							   "text", FM_LIST_MODEL_SIZE_COLUMN,
 							   NULL);
+	gtk_tree_view_column_set_resizable (column, TRUE);
 	gtk_tree_view_column_set_sort_column_id (column, FM_LIST_MODEL_SIZE_COLUMN);
 	gtk_tree_view_append_column (view->details->tree_view, column);
 
@@ -247,6 +300,7 @@ create_and_set_up_tree_view (FMListView *view)
 							   cell,
 							   "text", FM_LIST_MODEL_TYPE_COLUMN,
 							   NULL);
+	gtk_tree_view_column_set_resizable (column, TRUE);
 	gtk_tree_view_column_set_sort_column_id (column, FM_LIST_MODEL_TYPE_COLUMN);	
 	gtk_tree_view_append_column (view->details->tree_view, column);
 
@@ -256,6 +310,7 @@ create_and_set_up_tree_view (FMListView *view)
 							   cell,
 							   "text", FM_LIST_MODEL_DATE_MODIFIED_COLUMN,
 							   NULL);
+	gtk_tree_view_column_set_resizable (column, TRUE);
 	gtk_tree_view_column_set_sort_column_id (column, FM_LIST_MODEL_DATE_MODIFIED_COLUMN);
 	gtk_tree_view_append_column (view->details->tree_view, column);
 
@@ -294,7 +349,7 @@ set_sort_order_from_metadata_and_preferences (FMListView *list_view)
 
 	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (list_view->details->model),
 					      sort_column_id,
-					      sort_reversed ? GTK_SORT_ASCENDING : GTK_SORT_DESCENDING);
+					      sort_reversed ? GTK_SORT_DESCENDING : GTK_SORT_ASCENDING);
 }
 
 static void
@@ -413,7 +468,7 @@ fm_list_view_reset_to_defaults (FMDirectoryView *view)
 	gtk_tree_sortable_set_sort_column_id
 		(GTK_TREE_SORTABLE (FM_LIST_VIEW (view)->details->model),
 		 fm_list_model_get_sort_column_id_from_sort_type (default_sort_order_auto_value),
-		 default_sort_reversed_auto_value ? GTK_SORT_ASCENDING : GTK_SORT_DESCENDING);
+		 default_sort_reversed_auto_value ? GTK_SORT_DESCENDING : GTK_SORT_ASCENDING);
         fm_directory_view_restore_default_zoom_level (view);
 }
 
@@ -492,6 +547,9 @@ fm_list_view_finalize (GObject *object)
 
 	list_view = FM_LIST_VIEW (object);
 
+	eel_preferences_remove_callback (NAUTILUS_PREFERENCES_CLICK_POLICY,
+					 click_policy_changed, list_view);
+
 	g_free (list_view->details);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -541,7 +599,7 @@ fm_list_view_class_init (FMListViewClass *class)
         fm_directory_view_class->emblems_changed = fm_list_view_emblems_changed;
 	fm_directory_view_class->sort_directories_first_changed = fm_list_view_sort_directories_first_changed;
 	fm_directory_view_class->start_renaming_file = fm_list_view_start_renaming_file;
-	
+
 	eel_preferences_add_auto_enum (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_SORT_ORDER,
 				       (int *) &default_sort_order_auto_value);
 	eel_preferences_add_auto_boolean (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_SORT_IN_REVERSE_ORDER,
