@@ -45,7 +45,9 @@
 #include <eel/eel-string.h>
 #include <eel/eel-vfs-extensions.h>
 #include <eel/eel-xml-extensions.h>
+#include <librsvg/rsvg.h>
 #include <libxml/parser.h>
+#include <gtk/gtkalignment.h>
 #include <gtk/gtkcolorseldialog.h>
 #include <gtk/gtkdnd.h>
 #include <gtk/gtkentry.h>
@@ -53,6 +55,7 @@
 #include <gtk/gtkfilesel.h>
 #include <gtk/gtkframe.h>
 #include <gtk/gtkhbox.h>
+#include <gtk/gtkimage.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkscrolledwindow.h>
 #include <gtk/gtkselection.h>
@@ -142,7 +145,6 @@ struct NautilusPropertyBrowserDetails {
 static void     nautilus_property_browser_class_init      (GtkObjectClass                *object_klass);
 static void     nautilus_property_browser_init            (GtkObject                     *object);
 static void     nautilus_property_browser_destroy               (GtkObject                     *object);
-static void     nautilus_property_browser_preferences_changed   (NautilusPropertyBrowser       *property_browser);
 static void     nautilus_property_browser_update_contents       (NautilusPropertyBrowser       *property_browser);
 static void     nautilus_property_browser_set_category          (NautilusPropertyBrowser       *property_browser,
 								 const char                    *new_category);
@@ -235,8 +237,8 @@ static void
 nautilus_property_browser_init (GtkObject *object)
 {
  	NautilusPropertyBrowser *property_browser;
- 	GtkWidget* widget, *temp_box, *temp_hbox, *temp_frame;
-	GtkWidget* temp_button;
+ 	GtkWidget *widget, *temp_box, *temp_hbox, *temp_frame, *temp_image;
+	GtkWidget *temp_button, *align;
 	GtkWidget *viewport;
 	char *temp_str;
 	
@@ -351,20 +353,30 @@ nautilus_property_browser_init (GtkObject *object)
   	
   	/* create the "done" button */
  	temp_button = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
+	GTK_WIDGET_SET_FLAGS (temp_button, GTK_CAN_DEFAULT);
 	eel_gtk_button_set_padding (GTK_BUTTON (temp_button), GNOME_PAD_SMALL);
 
 	gtk_widget_show(temp_button);
-	gtk_box_pack_end (GTK_BOX(property_browser->details->bottom_box), temp_button, FALSE, FALSE, GNOME_PAD_SMALL);  
+	gtk_box_pack_end (GTK_BOX(property_browser->details->bottom_box), temp_button, FALSE, FALSE, GNOME_PAD_SMALL);
+	gtk_widget_grab_default (temp_button);
+	gtk_widget_grab_focus (temp_button);
  	g_signal_connect_object (temp_button, "clicked", G_CALLBACK (done_button_callback), property_browser, 0);
   	
   	/* create the "add new" button */
   	property_browser->details->add_button = gtk_button_new ();
-	gtk_widget_show(property_browser->details->add_button);
-	
 	property_browser->details->add_button_label = gtk_label_new_with_mnemonic (_("_Add new..."));
-	gtk_widget_show(property_browser->details->add_button_label);
-	gtk_container_add (GTK_CONTAINER(property_browser->details->add_button),
-			   property_browser->details->add_button_label);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (property_browser->details->add_button_label),
+				       GTK_WIDGET (property_browser->details->add_button));
+
+	temp_image = gtk_image_new_from_stock (GTK_STOCK_ADD, GTK_ICON_SIZE_BUTTON);
+	temp_hbox = gtk_hbox_new (FALSE, 2);
+	align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+	gtk_box_pack_start (GTK_BOX (temp_hbox), temp_image, FALSE, FALSE, 0);
+	gtk_box_pack_end (GTK_BOX (temp_hbox), property_browser->details->add_button_label, FALSE, FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (property_browser->details->add_button), align);
+	gtk_container_add (GTK_CONTAINER (align), temp_hbox);
+	gtk_widget_show_all (property_browser->details->add_button);
+
 	eel_gtk_button_set_padding (GTK_BUTTON (property_browser->details->add_button), GNOME_PAD_SMALL);
 	gtk_box_pack_end (GTK_BOX(property_browser->details->bottom_box),
 			  property_browser->details->add_button, FALSE, FALSE, GNOME_PAD_SMALL);
@@ -374,19 +386,21 @@ nautilus_property_browser_init (GtkObject *object)
 	
 	/* now create the "remove" button */
   	property_browser->details->remove_button = gtk_button_new();
-	gtk_widget_show(property_browser->details->remove_button);
-	
-	/* FIXME: Using spaces to add padding is not good design. */
 	property_browser->details->remove_button_label = gtk_label_new_with_mnemonic (_("_Remove..."));	
-	gtk_widget_show(property_browser->details->remove_button_label);
-	gtk_container_add (GTK_CONTAINER(property_browser->details->remove_button),
-			   property_browser->details->remove_button_label);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (property_browser->details->add_button_label),
+				       GTK_WIDGET (property_browser->details->add_button));
+	temp_image = gtk_image_new_from_stock (GTK_STOCK_REMOVE, GTK_ICON_SIZE_BUTTON);
+	temp_hbox = gtk_hbox_new (FALSE, 2);
+	align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+	gtk_box_pack_start (GTK_BOX (temp_hbox), temp_image, FALSE, FALSE, 0);
+	gtk_box_pack_end (GTK_BOX (temp_hbox), property_browser->details->remove_button_label, FALSE, FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (property_browser->details->remove_button), align);
+	gtk_container_add (GTK_CONTAINER (align), temp_hbox);
+	gtk_widget_show_all (property_browser->details->remove_button);
+
 	eel_gtk_button_set_padding (GTK_BUTTON (property_browser->details->remove_button), GNOME_PAD_SMALL);
 	gtk_box_pack_end (GTK_BOX (property_browser->details->bottom_box),
-			  property_browser->details->remove_button,
-			  FALSE,
-			  FALSE,
-			  GNOME_PAD_SMALL);
+			  property_browser->details->remove_button, FALSE, FALSE, GNOME_PAD_SMALL);
 	
  	g_signal_connect_object (property_browser->details->remove_button, "clicked",
 				 G_CALLBACK (remove_button_callback), property_browser, 0);
@@ -396,11 +410,6 @@ nautilus_property_browser_init (GtkObject *object)
 	/* the actual contents are created when necessary */	
   	property_browser->details->content_frame = NULL;
 
-	/* add callback for preference changes */
-	eel_preferences_add_callback (NAUTILUS_PREFERENCES_CAN_ADD_CONTENT, 
-				      (EelPreferencesCallback) nautilus_property_browser_preferences_changed, 
-				      property_browser);
-	
 	/* add a callback for when the theme changes */
 	eel_preferences_add_callback (NAUTILUS_PREFERENCES_THEME, 
 				      nautilus_property_browser_theme_changed,
@@ -441,9 +450,6 @@ nautilus_property_browser_destroy (GtkObject *object)
 	
 	g_free (property_browser->details);
 	
-	eel_preferences_remove_callback (NAUTILUS_PREFERENCES_CAN_ADD_CONTENT,
-					 (EelPreferencesCallback) nautilus_property_browser_preferences_changed, 
-					 NULL);
 	eel_preferences_remove_callback (NAUTILUS_PREFERENCES_THEME,
 					 nautilus_property_browser_theme_changed,
 					 property_browser);
@@ -462,7 +468,7 @@ nautilus_property_browser_new (void)
 
 	browser = NAUTILUS_PROPERTY_BROWSER
 		(gtk_widget_new (nautilus_property_browser_get_type (), NULL));
-	
+
 	gtk_container_set_border_width (GTK_CONTAINER (browser), 0);
   	gtk_widget_show (GTK_WIDGET(browser));
 	
@@ -658,6 +664,18 @@ make_drag_image (NautilusPropertyBrowser *property_browser, const char* file_nam
 	
 	orig_pixbuf = gdk_pixbuf_new_from_file (image_file_name, NULL);
 	
+	if (orig_pixbuf == NULL) {
+		orig_pixbuf = rsvg_pixbuf_from_file_at_max_size (image_file_name,
+								 MAX_ICON_WIDTH, MAX_ICON_HEIGHT,
+								 NULL);
+	}
+	
+	g_free (image_file_name);
+
+	if (orig_pixbuf == NULL) {
+		return NULL;
+	}
+	
 	is_reset = eel_strcmp (file_name, RESET_IMAGE_NAME) == 0;
 	
 	if (strcmp (property_browser->details->category, "patterns") == 0) {
@@ -667,8 +685,6 @@ make_drag_image (NautilusPropertyBrowser *property_browser, const char* file_nam
 		g_object_unref (orig_pixbuf);
 	}
 
-	g_free (image_file_name);
-
 	return pixbuf;
 }
 
@@ -908,53 +924,56 @@ nautilus_property_browser_remove_element (NautilusPropertyBrowser *property_brow
 static GtkWidget*
 nautilus_emblem_dialog_new (NautilusPropertyBrowser *property_browser)
 {
-	GtkWidget *widget, *entry;
+	GtkWidget *widget;
 	GtkWidget *dialog;
 	GtkWidget *table = gtk_table_new(2, 2, FALSE);
 
-	dialog = gtk_dialog_new_with_buttons (_("Create a New Emblem:"), NULL, GTK_DIALOG_MODAL,
-					      GTK_STOCK_OK, GTK_RESPONSE_OK,
+	dialog = gtk_dialog_new_with_buttons (_("Create a New Emblem:"),
+					      GTK_WINDOW (property_browser), GTK_DIALOG_MODAL,
 					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					      GTK_STOCK_OK, GTK_RESPONSE_OK,
 					      NULL);
 
+	/* install the table in the dialog */	
+	gtk_widget_show (table);	
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), table, TRUE, TRUE, GNOME_PAD);
+	gtk_dialog_set_default_response (GTK_DIALOG(dialog), GTK_RESPONSE_OK);
+
 	/* make the keyword label and field */	
 	
-	widget = gtk_label_new(_("Keyword:"));
+	widget = gtk_label_new_with_mnemonic(_("_Keyword:"));
 	gtk_widget_show(widget);
 	gtk_table_attach(GTK_TABLE(table), widget, 0, 1, 0, 1, GTK_FILL, GTK_FILL, GNOME_PAD_SMALL, GNOME_PAD_SMALL);
 	
   	property_browser->details->keyword = gtk_entry_new ();
-	gtk_entry_set_max_length (GTK_ENTRY (property_browser->details->keyword), 24);
+	gtk_widget_grab_focus(property_browser->details->keyword);
 	gtk_widget_show(property_browser->details->keyword);
 	gtk_table_attach(GTK_TABLE(table), property_browser->details->keyword, 1, 2, 0, 1, GTK_FILL, GTK_FILL, GNOME_PAD_SMALL, GNOME_PAD_SMALL);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (widget),
+				       GTK_WIDGET (property_browser->details->keyword));
 
 	/* default image is the generic emblem */
 	g_free(property_browser->details->image_path);		
 	property_browser->details->image_path = g_build_filename (NAUTILUS_PIXMAPDIR, "emblems.png", NULL);
 	
 	/* set up a gnome icon entry to pick the image file */
-	widget = gtk_label_new(_("Image:"));
+	widget = gtk_label_new_with_mnemonic (_("_Image:"));
 	gtk_widget_show(widget);
 	gtk_table_attach(GTK_TABLE(table), widget, 0, 1, 1, 2, GTK_FILL, GTK_FILL, GNOME_PAD_SMALL, GNOME_PAD_SMALL);
 
 	property_browser->details->file_entry = gnome_icon_entry_new (NULL, _("Select an image file for the new emblem:"));
 	gnome_icon_entry_set_pixmap_subdir (GNOME_ICON_ENTRY(property_browser->details->file_entry),
 						DATADIR "/pixmaps");
-	gnome_icon_entry_set_icon (GNOME_ICON_ENTRY(property_browser->details->file_entry),
-					property_browser->details->image_path);
-	
+	gnome_icon_entry_set_filename (GNOME_ICON_ENTRY(property_browser->details->file_entry),
+				       property_browser->details->image_path);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (widget),
+				       GTK_WIDGET (property_browser->details->file_entry));
+
 	gtk_widget_show(property_browser->details->file_entry);
 	gtk_table_attach(GTK_TABLE(table), property_browser->details->file_entry, 1, 2, 1, 2, GTK_FILL, GTK_FILL, GNOME_PAD_SMALL, GNOME_PAD_SMALL);
 	
-	entry = gnome_icon_entry_gtk_entry (GNOME_ICON_ENTRY(property_browser->details->file_entry));
-	gtk_entry_set_text(GTK_ENTRY(entry), property_browser->details->image_path);
-	
-	/* install the table in the dialog */	
-	gtk_widget_show (table);	
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), table, TRUE, TRUE, GNOME_PAD);
-	gtk_dialog_set_default_response (GTK_DIALOG(dialog), GTK_RESPONSE_OK);
-	gtk_window_set_wmclass(GTK_WINDOW(dialog), "emblem_dialog", "Nautilus");
-	
+	gnome_icon_entry_gtk_entry (GNOME_ICON_ENTRY(property_browser->details->file_entry));
+	gnome_icon_entry_set_filename (GNOME_ICON_ENTRY (property_browser->details->file_entry), property_browser->details->image_path);	
 	return dialog;
 }
 
@@ -968,33 +987,34 @@ nautilus_color_selection_dialog_new (NautilusPropertyBrowser *property_browser)
 	GtkWidget *table = gtk_table_new(2, 2, FALSE);
 
 	dialog = gtk_dialog_new_with_buttons (_("Create a New Color:"), NULL, GTK_DIALOG_MODAL,
-					      GTK_STOCK_OK, GTK_RESPONSE_OK,
 					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					      GTK_STOCK_OK, GTK_RESPONSE_OK,
 					      NULL);
 
 	/* make the name label and field */	
 	
-	widget = gtk_label_new(_("Color name:"));
+	widget = gtk_label_new_with_mnemonic(_("Color _name:"));
 	gtk_widget_show(widget);
 	gtk_table_attach(GTK_TABLE(table), widget, 0, 1, 0, 1, GTK_FILL, GTK_FILL, GNOME_PAD_SMALL, GNOME_PAD_SMALL);
 	
   	property_browser->details->color_name = gtk_entry_new ();
-	gtk_entry_set_max_length (GTK_ENTRY (property_browser->details->color_name), 24);
+	gtk_widget_grab_focus (property_browser->details->color_name);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (widget), property_browser->details->color_name);
 	gtk_widget_show(property_browser->details->color_name);
 	gtk_table_attach(GTK_TABLE(table), property_browser->details->color_name, 1, 2, 0, 1, GTK_FILL, GTK_FILL, GNOME_PAD_SMALL, GNOME_PAD_SMALL);
 
 	/* default image is the generic emblem */
 	g_free(property_browser->details->image_path);
 		
-	widget = gtk_label_new(_("Color value:"));
+	widget = gtk_label_new_with_mnemonic(_("Color _value:"));
 	gtk_widget_show(widget);
 	gtk_table_attach(GTK_TABLE(table), widget, 0, 1, 1, 2, GTK_FILL, GTK_FILL, GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  
 	/* set up a gnome file entry to pick the image file */
 	property_browser->details->color_picker = gnome_color_picker_new ();
 	gtk_widget_show (property_browser->details->color_picker);
-	
-	gtk_widget_show(property_browser->details->color_picker);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (widget), property_browser->details->color_picker);
+
 	gtk_table_attach(GTK_TABLE(table), property_browser->details->color_picker, 1, 2, 1, 2, GTK_FILL, GTK_FILL, GNOME_PAD_SMALL, GNOME_PAD_SMALL);
 		
 	/* install the table in the dialog */
@@ -1092,7 +1112,7 @@ add_new_pattern (NautilusPropertyBrowser *property_browser)
 	} else {
 		property_browser->details->dialog = 
 			eel_gnome_icon_selector_new (_("Select an image file to add as a pattern:"),
-				DATADIR "/pixmaps/tiles/",
+				DATADIR "/nautilus/patterns/",
 				GTK_WINDOW (property_browser),
 				(EelIconSelectionFunction) add_pattern_to_browser,
 				property_browser);						   
@@ -1131,7 +1151,7 @@ add_color_to_file (NautilusPropertyBrowser *property_browser, const char *color_
 
 /* handle the OK button being pushed on the color selection dialog */
 static void
-add_color_to_browser (GtkWidget *widget, int which_button, gpointer *data)
+add_color_to_browser (GtkWidget *widget, gint which_button, gpointer *data)
 {
 	char *color_spec;
 	const char *color_name;
@@ -1186,7 +1206,7 @@ show_color_selection_window (GtkWidget *widget, gpointer *data)
 	
 	eel_add_weak_pointer (&property_browser->details->dialog);
 
-	g_signal_connect_object (property_browser->details->dialog, "clicked",
+	g_signal_connect_object (property_browser->details->dialog, "response",
 				 G_CALLBACK (add_color_to_browser), property_browser, 0);
 	gtk_window_set_position (GTK_WINDOW (property_browser->details->dialog), GTK_WIN_POS_MOUSE);
 	gtk_widget_show (GTK_WIDGET(property_browser->details->dialog));
@@ -1227,12 +1247,14 @@ add_new_color (NautilusPropertyBrowser *property_browser)
 static gboolean
 emblem_keyword_valid (const char *keyword)
 {
-	int index, keyword_length;
+	const char *p;
+	gunichar c;
+	
+	for (p = keyword; *p; p = g_utf8_next_char (p)) {
+		c = g_utf8_get_char (p);
 
-	keyword_length = strlen (keyword);
-	for (index = 0; index < keyword_length; index++) {
-		if (!g_ascii_isalnum (keyword[index])
-		    && !g_ascii_isspace (keyword[index])) {
+		if (!g_unichar_isalnum (c) &&
+		    !g_unichar_isspace (c)) {
 			return FALSE;
 		}
 	}
@@ -1377,7 +1399,7 @@ add_new_emblem (NautilusPropertyBrowser *property_browser)
 
 		eel_add_weak_pointer (&property_browser->details->dialog);
 
-		g_signal_connect_object (property_browser->details->dialog, "clicked",
+		g_signal_connect_object (property_browser->details->dialog, "response",
 					 G_CALLBACK (emblem_dialog_clicked), property_browser, 0);
 		gtk_window_set_position (GTK_WINDOW (property_browser->details->dialog), GTK_WIN_POS_MOUSE);
 		gtk_widget_show (GTK_WIDGET(property_browser->details->dialog));
@@ -1484,7 +1506,7 @@ element_clicked_callback (GtkWidget *image_table,
 		drag_types[0].target = "x-special/gnome-reset-background";	
 	}
 	
-	target_list = gtk_target_list_new (drag_types, G_N_ELEMENTS (drag_types));	
+	target_list = gtk_target_list_new (drag_types, G_N_ELEMENTS (drag_types));
 	nautilus_property_browser_set_dragged_file(property_browser, element_name);
 	
 	context = gtk_drag_begin (GTK_WIDGET (property_browser),
@@ -1492,7 +1514,8 @@ element_clicked_callback (GtkWidget *image_table,
 				  GDK_ACTION_MOVE | GDK_ACTION_COPY,
 				  event->button,
 				  event->event);
-
+	gtk_target_list_unref (target_list);
+	
 	/* compute the offsets for dragging */
 	if (strcmp (drag_types[0].target, "application/x-color")) {
 		/* it's not a color, so, for now, it must be an image */
@@ -1546,14 +1569,6 @@ strip_extension (const char* string_to_strip)
 	return result_str;
 }
 
-/* handle preferences changing by updating the browser contents */
-
-static void
-nautilus_property_browser_preferences_changed (NautilusPropertyBrowser *property_browser)
-{
-	nautilus_property_browser_update_contents (property_browser);
-}
-
 static void
 labeled_image_configure (EelLabeledImage *labeled_image)
 {
@@ -1878,7 +1893,7 @@ nautilus_property_browser_update_contents (NautilusPropertyBrowser *property_bro
  	xmlDocPtr document;
  	EelBackground *background;
 	GtkWidget *viewport;
-	gboolean show_buttons, got_categories;
+	gboolean got_categories;
 	char *name, *image, *type, *description, *display_name, *path, *mode;
 	const char *text;
 
@@ -1978,7 +1993,6 @@ nautilus_property_browser_update_contents (NautilusPropertyBrowser *property_bro
 
 	/* update the title and button */
 
-	show_buttons = eel_preferences_get_boolean (NAUTILUS_PREFERENCES_CAN_ADD_CONTENT);
 
 	if (property_browser->details->category == NULL) {
 		gtk_label_set_text (GTK_LABEL (property_browser->details->title_label), _("Select A Category:"));
@@ -1989,18 +2003,18 @@ nautilus_property_browser_update_contents (NautilusPropertyBrowser *property_bro
 		char *label_text;
 				
 		if (property_browser->details->remove_mode) {
-			text = _("Cancel Remove");
+			text = _("C_ancel Remove");
 		} else {
 			/* FIXME: Using spaces to add padding is not good design. */
 			switch (property_browser->details->category_type) {
 			case NAUTILUS_PROPERTY_PATTERN:
-				text = _("Add a New Pattern...");
+				text = _("_Add a New Pattern...");
 				break;
 			case NAUTILUS_PROPERTY_COLOR:
-				text = _("Add a New Color...");
+				text = _("_Add a New Color...");
 				break;
 			case NAUTILUS_PROPERTY_EMBLEM:
-				text = _("Add a New Emblem...");
+				text = _("_Add a New Emblem...");
 				break;
 			default:
 				text = NULL;
@@ -2011,13 +2025,10 @@ nautilus_property_browser_update_contents (NautilusPropertyBrowser *property_bro
 		/* enable the "add new" button and update it's name */		
 		
 		if (text != NULL) {
-			gtk_label_set_text (GTK_LABEL(property_browser->details->add_button_label), text);
-		}
-		if (show_buttons) {
-			gtk_widget_show (property_browser->details->add_button);
-		} else {
-			gtk_widget_hide (property_browser->details->add_button);
+			gtk_label_set_text_with_mnemonic (GTK_LABEL(property_browser->details->add_button_label), text);
 		}
+		gtk_widget_show (property_browser->details->add_button);
+
 			
 		if (property_browser->details->remove_mode) {
 
@@ -2053,7 +2064,7 @@ nautilus_property_browser_update_contents (NautilusPropertyBrowser *property_bro
 		}
 		
 		if (label_text) {
-			gtk_label_set_text (GTK_LABEL (property_browser->details->title_label), label_text);
+			gtk_label_set_text_with_mnemonic (GTK_LABEL (property_browser->details->title_label), label_text);
 		}
 		g_free(label_text);
 
@@ -2063,27 +2074,26 @@ nautilus_property_browser_update_contents (NautilusPropertyBrowser *property_bro
 		/* FIXME: Using spaces to add padding is not good design. */
 		switch (property_browser->details->category_type) {
 		case NAUTILUS_PROPERTY_PATTERN:
-			text = _("Remove a Pattern...");
+			text = _("_Remove a Pattern...");
 			break;
 		case NAUTILUS_PROPERTY_COLOR:
-			text = _("Remove a Color...");
+			text = _("_Remove a Color...");
 			break;
 		case NAUTILUS_PROPERTY_EMBLEM:
-			text = _("Remove an Emblem...");
+			text = _("_Remove an Emblem...");
 			break;
 		default:
 			text = NULL;
 			break;
 		}
 		
-		if (!show_buttons
-		    || property_browser->details->remove_mode
+		if (property_browser->details->remove_mode
 		    || !property_browser->details->has_local)
 			gtk_widget_hide(property_browser->details->remove_button);
 		else
 			gtk_widget_show(property_browser->details->remove_button);
 		if (text != NULL) {
-			gtk_label_set_text (GTK_LABEL(property_browser->details->remove_button_label), text);
+			gtk_label_set_text_with_mnemonic (GTK_LABEL(property_browser->details->remove_button_label), text);
 		}
 	}
 }
