@@ -21,7 +21,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  *  Authors: Elliot Lee <sopwith@redhat.com>
- *           Darin Adler <darin@eazel.com>
+ *           Darin Adler <darin@bentspoon.com>
  *
  */
 
@@ -42,12 +42,13 @@ typedef enum {
         NAUTILUS_LOCATION_CHANGE_REDIRECT
 } NautilusLocationChangeType;
 
-/* FIXME bugzilla.eazel.com 2575: Migrate more fields into here. */
+/* FIXME bugzilla.gnome.org 42575: Migrate more fields into here. */
 struct NautilusWindowDetails
 {
         /* Bonobo. */
         BonoboUIContainer *ui_container;
         BonoboUIComponent *shell_ui;
+        BonoboUIComponent *status_ui;
         gboolean updating_bonobo_state;
 
 	int ui_change_depth;
@@ -60,6 +61,7 @@ struct NautilusWindowDetails
 	guint refresh_go_menu_idle_id;
 
 	/* Toolbar. */
+        GtkTooltips *tooltips;
 	BonoboUIToolbarButtonItem *back_button_item;
 	BonoboUIToolbarButtonItem *forward_button_item;
 
@@ -82,7 +84,9 @@ struct NautilusWindowDetails
         NautilusViewIdentifier *extra_viewer;
 
         /* Throbber. */
-	Bonobo_EventSource_ListenerId throbber_location_change_request_listener_id;
+        gboolean           throbber_active;
+	Bonobo_PropertyBag throbber_property_bag;
+	Bonobo_Listener    throbber_listener;
 
         /* Deferred location change. */
         char *location_to_change_to_at_idle;
